"""Convert SlashDB configs from v0.8.x to v0.9.x."""

from passlib.apps import custom_app_context as pwd_context
import sys
import yaml


def save_to_file(cfg, path):
    with open(path, 'w+') as f:
        yaml.dump(cfg, f)


def convert_user_cfg(path='userds.cfg'):
    with open(path, 'r') as f:
        configs = yaml.load(f.read())

        for config_id, config in configs.items():
            try:
                user_id = config['user_id']
            except KeyError:
                user_id = config_id
                config['user_id'] = config_id
                print '%s: added user_id key'

            old_password = config['password']
            new_password = pwd_context.encrypt('IlVF7PrZsz' + old_password)
            config['password'] = new_password
            print '%s : %s : %s' % (user_id, old_password, new_password)

        return configs


def convert_database_cfg(path='databases.cfg'):
    with open(path, 'r') as f:
        configs = yaml.load(f.read())
        for config_id, config in configs.items():
            try:
                db_id = config['db_id']
            except KeyError:
                db_id = config_id
                config['db_id'] = config_id
                print '%s: added db_id key'

            if 'autoload_user' not in config:
                config['autoload_user'] = config['sysuser']
                print '%s: sysuser -> autoload_user' % db_id

        return configs


def convert_querydef_cfg(path='querydefs.cfg'):
    with open(path, 'r') as f:
        configs = yaml.load(f.read())
        for config_id, config in configs.items():
            try:
                config['db_id']
            except KeyError:
                config['query_id'] = config_id
                print '%s: added query_id key'

        return configs


if __name__ == '__main__':

    print sys.argv
    cfg_type = sys.argv[1]
    in_path = sys.argv[2]
    out_path = sys.argv[3]

    if cfg_type == 'users':
        configs = convert_user_cfg(in_path)
        save_to_file(configs, out_path)

    if cfg_type == 'databases':
        configs = convert_database_cfg(in_path)
        save_to_file(configs, out_path)

    if cfg_type == 'querydefs':
        configs = convert_querydef_cfg(in_path)
        save_to_file(configs, out_path)

