import ConfigParser
import argparse
import re
import telnetlib
import memcache


class MemcachedStats:
    """https://github.com/abstatic/python-memcached-stats/blob/master/src/memcached_stats.py"""

    _client = None
    _key_regex = re.compile(ur'ITEM (.*) \[(.*); (.*)\]')
    _slab_regex = re.compile(ur'STAT items:(.*):number')
    _stat_regex = re.compile(ur"STAT (.*) (.*)\r")

    def __init__(self, host='localhost', port='11211'):
        self._host = host
        self._port = port

    @property
    def client(self):
        if self._client is None:
            self._client = telnetlib.Telnet(self._host, self._port)
        return self._client

    def command(self, cmd):
        """Write a command to telnet and return the response."""
        self.client.write("%s\n" % cmd)
        return self.client.read_until('END')

    def key_details(self, sort=True, limit=100):
        """Return a list of tuples containing keys and details."""
        cmd = 'stats cachedump %s %s'
        keys = [key for id in self.slab_ids()
                for key in self._key_regex.findall(self.command(cmd % (id, limit)))]
        if sort:
            return sorted(keys)
        else:
            return keys

    def keys(self, sort=True, limit=100):
        """Return a list of keys in use."""
        return [key[0] for key in self.key_details(sort=sort, limit=limit)]

    def slab_ids(self):
        """Return a list of slab ids in use."""
        return self._slab_regex.findall(self.command('stats items'))

    def stats(self):
        """Return a dict containing memcached stats."""
        return dict(self._stat_regex.findall(self.command('stats')))


def main(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument('command', help='a command like "clean-memcache"')
    parser.add_argument('ini_file', help='path to the slashdb.ini file')

    args = parser.parse_args()
    if args.command == 'clean-memcache':
        try:
            print 'Parsing ini file %s' % args.ini_file
            ini_config = ConfigParser.ConfigParser()
            ini_config.readfp(open(args.ini_file))
            mc_server = ini_config.get('app:sdbapp', 'memcached_server')
            mc_root = ini_config.get('app:sdbapp', 'memcache_root_key')
        except (IOError, IndexError, ConfigParser.Error) as err:
            print "Failed parsing config file: %s" % err
        else:
            mcstats = MemcachedStats(*mc_server.split(':'))

            def predicate(v):
                return v.startswith(mc_root)

            mc_keys = filter(predicate, mcstats.keys())
            mc = memcache.Client([mc_server])
            mc.delete_multi(mc_keys)
            print 'Deleted memcache keys: %s' % mc_keys


if __name__ == "__main__":
    main()
