// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
        .controller('DBConfigModalCtrl', ['$window', '$timeout', '$q', '$rootScope', '$scope', '$uibModalInstance', 'slashDB', 'configId', 'selectizeService',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance, slashdb, configId, selectizeService) {
            $scope.newDbConfig = !Boolean(configId);
            $scope.configId = configId;
            $scope.deleteVisible = !$scope.newDbConfig;
            $scope.username = slashdb.settings.user;
            // stores values only for local use, not to be sent to slashdb
            $scope.dbtypes = slashdb.settings.dbtypes;
            var allCharsets = [];
            var mysqlCharsets = [];

            slashdb.settings.py_encodings.forEach(function(charset){
                allCharsets.push(charset.label);
                if (charset.mysql) mysqlCharsets.push(charset.label)
            });

            $scope.required_fields = {};

            $scope.getCharsets = function(dbType){
                if (dbType == 'mysql') {
                    return mysqlCharsets;
                } else {
                    return allCharsets;
                }
            };

            /** @type {{[a: string]: (string | boolean) }} */
            var initExtraConfigData = {
                disabledId: false,
                host: '',
                port: '',
                name: '',
                override: false,
            };

            var init = function () {
                $scope.selectizeService = selectizeService;
                selectizeService.updateUserOptions(null);
                $scope.okVisible = false;
                $scope.modalAlert = '';
                resetDbConfig();
                getDbConfig();
            };

            /** @returns {Promise} */
            var getDbConfig = function () {
                if ($scope.newDbConfig) {
                    return $q(function (resolve, reject) {
                        resolve();
                    });
                }

                return $q(function (resolve, reject) {
                    slashdb.get('/dbdef/' + $scope.configId + '.json', {
                            params: {
                                guidata: ''
                            }
                        }, false, false)
                        .then(function (response) {
                            $scope.configData = response.data;
                            $scope.extraConfigData.disabledId = true;
                            $scope.deleteVisible = $scope.configData.editable;
                            selectizeService.updateUserOptions($scope.configData);
                            var dbEncoding = $scope.configData.db_encoding;
                            if ($scope.isSQLAlchemy()) updateExtraConfigData();
                            if ($scope.configData.db_type == 'mysql'){
                                if (!mysqlCharsets.includes(dbEncoding)) mysqlCharsets.push(dbEncoding);
                            } else {
                                if (!allCharsets.includes(dbEncoding)) allCharsets.push(dbEncoding);
                            }
                            // refresh slashDB.settings
                            slashdb.setupSettings();
                            resolve();
                        });
                });
            };

            var resetDbConfig = function () {
                // stores database definition model, some initial values
                $scope.configData = {
                    db_type: 'sqlite',
                    autoload: false,
                    autoconnect: true,
                    editable: true,
                    autoload_user: {dbuser: null, dbpass: null},
                };
                // @ts-ignore
                $scope.extraConfigData = angular.extend({}, initExtraConfigData);
            };

            var updateExtraConfigData = function (override) {
                // Updates $scope.extraConfigData to display gui correctly.
                var parser = $window.document.createElement('a');
                parser.href = 'http://' + $scope.configData.connection; // hack for dummy parser that does not know database protocols
                if (!override) {
                    var port = parser.port ? ':' + parser.port : '';
                    override = $scope.configData.connection !== parser.hostname + port + '/' + parser.pathname.slice(1);
                }
                $scope.extraConfigData = {
                    disabledId: $scope.extraConfigData.disabledId,
                    host: parser.hostname,
                    port: parser.port,
                    name: parser.pathname.slice(1),
                    override: override
                };
            };

            /** @returns {boolean} */
            $scope.requiresCredentials = function () {
                return $scope.configData.use_user_db_credentials_for_mapping &&
                    $scope.configData.db_type !== 'sqlite' &&
                    $scope.configData.autoload &&
                    $scope.isSQLAlchemy();
            };

            $scope.isSQLAlchemy = function () {
                return ['mssql', 'oracle', 'mysql', 'sqlite', 'postgresql', 'db2'].includes($scope.configData.db_type);
            };

            $scope.updateConnectionString = function () {
                if ($scope.extraConfigData.override) {
                    updateExtraConfigData($scope.extraConfigData.override);
                } else {
                    var port = $scope.extraConfigData.port ? ':' + $scope.extraConfigData.port : '';
                    var host = $scope.extraConfigData.host ? $scope.extraConfigData.host : '';
                    var name = $scope.extraConfigData.name ? $scope.extraConfigData.name : '';
                    if (host || port || name) {
                        $scope.configData.connection = host + port + '/' + name;
                    }
                    else {
                        $scope.configData.connection = null;
                    }
                }

            };
            $scope.delete = function () {
                slashdb.deleteDBDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    });
            };

            /**
             * @param {string} dbId
             * @param {boolean} useForMapping
             */
            var sendAddBtnMsg = function (dbId, useForMapping) {
                var addBtnMsg = useForMapping ? '' :
                    'Click to associate this user with database "' + dbId + '" login / password.';
                $rootScope.$broadcast('open-user-settings-modal', addBtnMsg);
            };

            var fieldMappingDbDef = {
                'autoconnect': 'Connect Automatically',
                'autoload': 'Auto Discover',
                'connection': 'Database File Location',
                'db_encoding': 'Database Charset',
                'db_id': 'Database ID',
                'db_type': 'Type',
                'desc': 'Description',
                'use_user_db_credentials_for_mapping': 'Use Credentials For Current User',
                'read': 'View DB Config',
                'write': 'Modify DB config',
                'execute': 'Connect / Disconnect DB',
            };

            var setInvalidFields = function(errors) {
                var invalid = [];
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'INVALID') {
                            if (err['field'] === 'connection' &&
                                $scope.configData.db_type === 'sqlite' &&
                                err['description'] === "u'' is too short") {
                                // not invalid, but missing
                                return
                            }
                            invalid.push(err['field']);

                            if (err['field'] === 'connection' &&
                                err['sub_field_type'] === 'INVALID') {
                                invalid.push(err['sub_field'])
                            }
                        }
                    }
                );

                invalid.forEach(function (value) {
                    if (!$scope.configForm.hasOwnProperty(value)) {
                        console.log('Warning - bad property ' + value)
                        return
                    }
                    $scope.configForm[value].$setValidity('valid', false)
                });
                return invalid
            };

            var setRequiredFields = function (errors) {
                var fields = ['db_id', 'db_type', 'db_encoding', 'db_name', 'db_host',]
                var required = {};
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'MISSING') {
                            required[err['field']] = true;
                        }

                        if (err['field'] === 'connection') {
                            // when conn str is missing or too short
                            if (!err.hasOwnProperty('sub_field') &&
                                $scope.configData.db_type !== 'sqlite') {
                                required['db_name'] = true;
                                required['db_host'] = true;
                            } // when conn str failed class validation
                            else if(err['sub_field_type'] === 'MISSING') {
                                required[err['sub_field']] = true;
                            }
                            else if ($scope.configData.db_type === 'sqlite' &&
                                err['description'] === "u'' is too short") {
                                // not invalid, but missing
                                required[err['field']] = true;
                            }
                        }
                    }
                );

                fields.forEach(function (value) {
                    required[value] = true;
                });

                $scope.required_fields = required;
            };

            var getUserFriendlyErrorMessages = function (errors) {
                return 'Errors found in incoming database definition: ' +
                    errors.map(
                    function (err, _, __) {
                        if (err['type'] === 'MISSING') {
                            if (err['field'] !== 'connection') {
                                return fieldMappingDbDef[err['field']] +
                                    ' is obligatory'
                            }

                            if ($scope.configData.db_type === 'sqlite') {
                                return 'Database File Location is obligatory'
                            }

                            // connection
                            return 'Database Host, Name or ' +
                                'Connection String is obligatory'
                        }

                        if (err['type'] === 'EXTRA') {
                            return err['description'] + " - try to reload page"
                        }

                        if (err['field'] === 'db_id' &&
                            err['description'].includes('already exists')) {
                            return 'Database already exists'
                        }

                        if (err['field'] !== 'connection') {
                            return fieldMappingDbDef[err['field']] + ' is invalid'
                        }

                        if ($scope.configData.db_type === 'sqlite' &&
                            err['field'] === 'connection') {
                            return 'Database File Location is obligatory'
                        }

                        if (err['field'] === 'connection') {
                            if (err['description'] === "None is not of type 'string'") {
                                return 'Database Host, Name or ' +
                                'Connection String is obligatory'
                            }
                            return err['description']
                        }

                    }
                ).join(', ') + '.'
            };

            var resetErrors = function() {
                $scope.modalAlert = '';
                $scope.required_fields = {};
                for (var key in $scope.configForm) {
                    var value = $scope.configForm[key];
                    if (typeof value === 'object'
                        && value.hasOwnProperty('$modelValue')) {
                        value.$setValidity('valid', true)
                    }
                }
            };

            $scope.resetError = function (field) {
                $scope.configForm[field].$setValidity('valid', true)
            };

            var setErrors = function(errors) {
                $scope.modalAlert = getUserFriendlyErrorMessages(errors);
                setInvalidFields(errors);
                setRequiredFields(errors);
            };

            var getPayload = function(raw_data) {
                var payload = Object.assign({}, raw_data);
                delete payload.editable;
                delete payload.viewable;
                delete payload.executable;
                delete payload.connect_status;

                return payload
            };

            $scope.save = function () {
                $scope.configData.override_switch = $scope.extraConfigData.override;

                var payload = getPayload($scope.configData);

                resetErrors();
                if ($scope.newDbConfig) {
                    slashdb.createDBDef(payload)
                        .then(function (response) {
                            /** @type {boolean} */
                            var useForMapping = $scope.configData.use_user_db_credentials_for_mapping || false;

                            blinkOK();
                            $scope.newDbConfig = false;
                            $scope.configId = $scope.configData.db_id;
                            getDbConfig().then(function () {
                                sendAddBtnMsg($scope.configId, useForMapping);
                            });
                            setRequiredFields([]);
                        })
                        .catch(function (response) {
                            setErrors(response.data.errors);
                        });
                } else {
                    slashdb.updateDBDef($scope.configData.db_id, payload)
                        .then(function (response) {
                            blinkOK();
                            getDbConfig();
                            setRequiredFields([]);
                        })
                        .catch(function (response) {
                            setErrors(response.data.errors);
                        });
                }
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            var blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };
            init();
        }
    ]);