from slashdb.constants import regex_good_name_str

config_schema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "description": "general config structure",
    "additionalProperties": False,
    "patternProperties": {
        regex_good_name_str: {
            "type": "object",
            "description": "single config object",
        }
    }
}

dbdef_schema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "description": "database",
    "additionalProperties": False,
    "properties": {
        "alternate_key": {
            "oneOf": [{
                "type": "object",
                "patternProperties": {
                    regex_good_name_str: {
                        "oneOf": [
                            {
                                "type": "array",
                                "minItems": 1,
                                "uniqueItems": True,
                                "items": {
                                    "type": "string",
                                    "minLength": 1
                                }
                            },
                            {"type": "null"}
                        ]
                    }
                },
            },
                {"type": "null"}
            ]
        },
        "autoconnect": {
            "type": "boolean"
        },
        "autoload": {
            "type": "boolean"
        },
        "autoload_user": {
            "oneOf": [
                {
                    "type": "object",
                    "additionalProperties": False,
                    "properties": {
                        "dbpass": {
                            "type": ["string", "null"]
                        },
                        "dbuser": {
                            "type": ["string", "null"]
                        }
                    },
                    "required": [
                        "dbuser",
                        ]
                },
                {
                    "type": "object",
                    "additionalProperties": False,
                    "properties": {}
                },
                {"type": "null"}
            ]
        },
        "connection": {
            "type": "string",
            "minLength": 1
        },
        "creator": {
            "type": ["string", "null"],
            "pattern": regex_good_name_str
        },
        "db_encoding": {
            "type": "string",
            "minLength": 1
        },
        "db_id": {
            "type": "string",
            "pattern": regex_good_name_str
        },
        "db_schema": {
            "type": ["string", "null"]
        },
        "db_type": {
            "type": "string",
            "minLength": 1
        },
        "desc": {
            "type": ["string", "null"]
        },
        "excluded_columns": {
            "oneOf": [
                {
                    "type": "object",
                    "patternProperties": {
                        regex_good_name_str: {
                            "type": "array",
                            "uniqueItems": True,
                            "items": {
                                "type": "string",
                                "minLength": 1
                            }
                        }
                    },
                },
                {"type": "null"}
            ]
        },
        "execute": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str,
                    }
                },
                {"type": "null"},
            ]
        },
        "foreign_keys": {
            "oneOf": [
                {"type": "object"},
                {"type": "null"},
            ]
        },
        "read": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str,
                    }
                },
                {"type": "null"},
            ]
        },
        "write": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str,
                    }
                },
                {"type": "null"},
            ]
        },
        "use_user_db_credentials_for_mapping": {
            "type": ["boolean", "null"]
        },
        "override_switch": {
            "type": ["boolean", "null"]
        }
    },
    "required": [
        "autoconnect",
        "autoload",
        "connection",
        "db_encoding",
        "db_id",
        "db_type"
    ],
    "optional": [
        "autoload_user",
        "alternate_key",
        "db_schema",
        "desc",
        "excluded_columns",
        "execute",
        "foreign_keys",
        "read",
        "write",
        "use_user_db_credentials_for_mapping",
        'override_switch',
    ],
}

userdef_schema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "additionalProperties": False,
    "properties": {
        "api_key": {
            "type": ["string", "null"]
        },
        "creator": {
            "type": ["string", "null"],
            "pattern": regex_good_name_str
        },
        "databases": {
            "oneOf": [{
                "type": "object",
                "patternProperties": {
                    regex_good_name_str: {
                        "oneOf": [
                            {
                                "type": "object",
                                "additionalProperties": False,
                                "properties": {
                                    "dbpass": {
                                        "type": ["string", "null"]
                                    },
                                    "dbuser": {
                                        "type": ["string"]
                                    }
                                },
                                "required": [
                                    "dbuser",
                                ]
                            },
                            {
                                "type": "object",
                                "additionalProperties": False,
                                "properties": {}
                            },
                            {"type": "null"}
                        ]
                    }
                },
            },
                {"type": "null"}
            ]
        },
        "dbdef": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "enum": ["view", "create"]
                    },
                    "additionalItems": False,
                    "uniqueItems": True
                },
                {"type": "null"}
            ]
        },
        "edit": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str,
                    }
                },
                {"type": "null"}
            ]
        },
        "email": {
            "type": ["string", "null"]
        },
        "name": {
            "type": ["string", "null"]
        },
        "password": {
            "type": ["string", "null"]
        },
        "querydef": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "enum": ["view", "create"]
                    },
                    "additionalItems": False,
                    "uniqueItems": True
                },
                {"type": "null"}
            ]
        },
        "user_id": {
            "type": "string",
            "pattern": regex_good_name_str
        },
        "userdef": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "enum": ["view", "create"]
                    },
                    "additionalItems": False,
                    "uniqueItems": True
                },
                {"type": "null"}
            ]
        },
        "view": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str
                    }
                },
                {"type": "null"}
            ]
        }
    },
    "required": [
        "user_id",
    ],
    "optional": [
        "api_key",
        "creator",
        "databases",
        "edit",
        "email",
        "name",
        "password",
        "querydef",
        "dbdef",
        "userdef",
        "view",
    ]
}

querydef_schema = {
    "$schema": "http://json-schema.org/draft-07/schema#",
    "type": "object",
    "additionalProperties": False,
    "properties": {
        "creator": {
            "type": ["string", "null"],
            "pattern": regex_good_name_str
        },
        "database": {
            "type": "string",
            "pattern": regex_good_name_str,
        },
        "desc": {
            "type": ["string", "null"]
        },
        "execute": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str,
                    }
                },
                {"type": "null"}
            ]
        },
        "http_methods": {
            "type": "object",
            "additionalProperties": False,
            "minProperties": 1,
            "properties": {
                "DELETE": {
                    "type": "boolean"
                },
                "GET": {
                    "type": "boolean"
                },
                "POST": {
                    "type": "boolean"
                },
                "PUT": {
                    "type": "boolean"
                },
            },
        },
        "query_id": {
            "type": "string",
            "pattern": regex_good_name_str
        },
        "read": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str
                    }
                },
                {"type": "null"}
            ]
        },
        "sqlstr": {
            "type": "string",
            "minLength": 1
        },
        "write": {
            "oneOf": [
                {
                    "type": "array",
                    "items": {
                        "type": "string",
                        "pattern": regex_good_name_str
                    }
                },
                {"type": "null"}
            ]
        },
    },
    "required": [
        "query_id",
        "database",
        "http_methods",
        "sqlstr",
    ],
    "optional": [
        "creator",
        "desc",
        "execute",
        "read",
        "write",
    ]
}
